@@ -12,7 +12,7 @@
 
 // The columns in the DOM Tree View
 enum DomModelColumns {
-	DOM_COL_XML_PATH,
+	DOM_COL_XML_POINTER,
 	DOM_COL_ICON,
 	DOM_COL_ELEMENT_NAME,
 	DOM_COL_ID_NAME,
@@ -24,6 +24,8 @@ typedef enum DomModelColumns DomModelColumnsEnum;
 // Public prototypes
 void xacobeo_populate_gtk_text_buffer (GtkTextBuffer *buffer, xmlNode *node, HV *namespaces);
 void xacobeo_populate_gtk_tree_store  (GtkTreeStore *store,   xmlNode *node, HV *namespaces);
+gchar* xacobeo_get_node_path          (xmlNode *node, HV *namespaces);
+gchar* xacobeo_get_node_mark          (xmlNode *node);
 
 
 #endif
